# A place to centralize design tokens to simplify tweaking the appearance of the CLI
# See https://rich.readthedocs.io/en/stable/appendix/colors.html


HEADER_COLOR = "cornflower_blue"
HEADER_COLOR_HEX = "#5f87ff"
PRIMARY_COL = "default"
MUTED_COL = "grey46"
WARNING = "yellow"
ACTIVE_INDICATOR = "magenta"
